using System.Threading.Tasks;
using FluentAssertions;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using NSubstitute;
using Robware.Api.Auth.Controllers;
using Robware.Api.Auth.Models;
using Robware.Auth;
using Xunit;

namespace Robware.Api.Auth.Tests {
	public class AuthControllerTests {
		private class TestUser : User {
			public TestUser(string username, string password) {
				Username = username;
				Password = password;
			}
		}

		[Fact]
		public async Task Authenticate_WithSuccessfulLoginRequest_ReturnsUser() {
			var logger = Substitute.For<ILogger<AuthController>>();
			var authenticator = Substitute.For<IAuthenticator>();
			authenticator.Authenticate("username", "password").Returns((AuthenticationResult.Success, new TestUser("username", "password")));

			var request = new LoginRequest {
				Username = "username",
				Password = "password"
			};

			var expectation = new TestUser("username", "password");

			var controller = new AuthController(logger, authenticator);
			(await controller.Authenticate(request)).Value.Should().BeEquivalentTo(expectation);
		}

		[Fact]
		public async Task Authenticate_WithIncorrectPassword_Returns401() {
			var logger = Substitute.For<ILogger<AuthController>>();
			var authenticator = Substitute.For<IAuthenticator>();
			authenticator.Authenticate("username", "password").Returns((AuthenticationResult.IncorrectPassword, null));

			var request = new LoginRequest {
				Username = "username",
				Password = "password"
			};

			var controller = new AuthController(logger, authenticator);
			(await controller.Authenticate(request)).Result.Should().BeOfType<UnauthorizedResult>();
		}

		[Fact]
		public async Task Authenticate_WithIncorrectPassword_Returns404() {
			var logger = Substitute.For<ILogger<AuthController>>();
			var authenticator = Substitute.For<IAuthenticator>();
			authenticator.Authenticate("username", "password").Returns((AuthenticationResult.NotFound, null));

			var request = new LoginRequest {
				Username = "username",
				Password = "password"
			};

			var controller = new AuthController(logger, authenticator);
			(await controller.Authenticate(request)).Result.Should().BeOfType<NotFoundResult>();
		}
	}
}
